;********************************************************************
;Polls a 16 key XY matrix keypad for a key press.
;The keypad is prganised as follows...
;      __________________________________
;     |    1   |    2   |   3   |   F    |
;     |	(X1/Y1)| (X2/Y1)|(X3/Y1)|(X4/Y1) |
;     |--------|--------|-------|--------| 
;     |    4   |    5   |   6   |   E	 |
;     | (X1/Y2)| (X2/Y2)|(X3/Y2)|(X4/Y2) |
;     |-----------------|-------|--------|
;     |    7   |    8   |   9   |   D    | 
;     | (X1/Y3)| (X2/Y3)|(X3/Y3)|(X4/Y3) |
;     |--------|--------|-------|--------|
;     |    A   |    0   |   B   |   C    |
;     | (X1/Y4)| (X2/Y4)|(X3/Y4)|(X4/Y4) |
;     |__________________________________|
;The keypad used for this program has a matrix connection as follows...
;       
;     ~                                  ~
;     |                                  | 
;     |      Y4 Y3 Y2 Y2 X4 X3 X2 X1     | 
;     |       X  X  X  X  X  X  X  X     |
;     |__________________________________|
; Looking at the bottom rear of the keypadpad    
;
;The connectivity to the ADuC812 is as follows...
;  ROws are the Ys, the Columns are the Xs
;  Row1...Y1 -> P2.0
;  Row2...Y2 -> P2.1
;  Row3...Y3 -> P2.2
;  Row4...Y4 -> P2.3
;  Col1...X1 -> P2.4
;  Col2...X2 -> P2.5
;  Col3...X3 -> P2.6
;  Col4...X4 -> P2.7
;
;Poll all rows...P2.0, P2.1, P2.2 & P2.3 by driving a 0 on the line and checking for
;the status of each column..P2.4, P2.5, P2.6 & P2.7.
;
;Schematic
;---------
;			Vdd
;                       ---
;                        |
;                        |___________________________ 
;                          |      |       |       |   
;                          |      |       |       |
;                          \      \       \       \
;                          /      /       /       /
;                          \10k   \10k    \10k    \10k
;                          /      /       /       /
;                          |      |       |       |
;                          |      |       |       |
;                          X1     X2      X3      X4
;
;Operation:							
;----------
;The columns X1..X4 are pulled high through the 10k resistors.
;For each poll each row is driven low from the uC and a check done 
;on each column in turn to see which column if any has been pulled low.
;The column that has been pulled low has had that particular switch at 
;the interface between the row that has been driven low and the column 
;that is now reading low. 
;
;
;Note:
;-----
;This program uses a 100ms DELAY to protect against key debounce
;It does offer some protection against continuous key depress inside 
;the key subroutines.
;
;UART Timing.
;Using T1 for Baud Rate Generation. Mclk = 11.0592MHz
;Baud Rate = [(2^SMOD)/32]*[(Mclk)/{12*(256-TH1)}]
;       if Baud Rate=9600 & SMOD=0     
;       TH1 reload value turns out to be -3 
;i.e.  [(2^0)/32]*[11.0592E6/{12*3}] = 9600
;********************************************************************

$MOD812                         ;use ADuC812 predefined symbols

;Definitions
Row1    EQU     P2.0
Row2    EQU     P2.1
Row3    EQU     P2.2
Row4    EQU     P2.3
Col1    EQU     P2.4
Col2    EQU     P2.5
Col3    EQU     P2.6
Col4    EQU     P2.7


CSEG

ORG  0000h
	JMP Main

ORG 0100h
MAIN:   MOV SCON, #50H  ;CONFIGURE FOR MODE 1
        MOV PCON, #00H  ;Clr SMOD 
        MOV TMOD, #20H  ;SET T1 TO 8 BIT AUTO-RELOAD MODE 
        MOV TH1, #-3    ;RELOAD VALUE FOR 9600 BAUD
        SETB TR1        ;START TIMER 1 RUNNING
	
keys:   
        Mov P2, #0FFh   ;Set all P2 lines high initially
;Row 1 (keys 1, 2, 3, F) check
	CLR Row1        ;Drive a 0 onto Row1 (Y1)
One:	JNB Col1, k1    ;Check for Hi on Col1,if Low then Key1 is pressed
	Jmp Two         ;If key1 not pressed, skip to next key
k1:	call key1       ;If Key 1 pressed run subroutine Key1
Two:	JNB Col2, K2    ;Check for Hi on Col2, If Low then Key2 is pressed
	Jmp three       ;If Key2 not pressed skip to next key
k2:	call Key2       ;If key2 pressed run subroutine key2
three:	JNB Col3, K3    ;Check for Hi on Col3, If Low then Key3 is pressed
	JMP Four        ;If key3 not pressed skip to next key
K3:	Call Key3       ;If key 3 pressed run subroutine Key3
four:   JNB Col4, KF    ;Check for Hi on Col4, If Low then KeyF is pressed
	JMP Five        ;If KeyF not pressed skip to next key
kF:	Call KeyF       ;If KeyF is pressed run subroutine KeyF
five:	SETB Row1       ;Reinitialise Row1

;Row 2 (Keys 4, 5, 6, E) check
        CLR Row2        ;Drive Row2 low (Y2)
six:    JNB Col1, k4    ;Check for Hi on Col1, if Low then Key4 is pressed
        Jmp seven       ;If key4 not pressed skip to next key
K4:     Call Key4       ;If Key4 is pressed run subroutine Key4
seven:  JNB Col2, K5    ;Check for Hi on Col2, If Low then Key5 is pressed
        Jmp eight       ;If Key5 not pressed skip to next key
K5:     Call Key5       ;If Key5 is pressed run subroutine Key5
eight:  JNB Col3, K6    ;Check for Hi on Col3, If Low then Key6 is pressed
        Jmp nine        ;If Key 6 not pressed skip to next key
K6:     Call Key6       ;If key6 is pressed run subroutine Key6
nine:   JNB Col4, KE    ;Check for Hi on Col4, If Low then KeyE is pressed
        Jmp ten         ;If key E not pressed skip to next key
KE:     Call KeyE       ;If Key E is pressed run subroutine KeyE
ten:    SETB Row2       ;Reinitialise Row2

;Row 3 (Keys 7, 8, 9, D) check
        CLR Row3        ;Drive Row3 Low (Y3)
eleven: JNB Col1, k7    ;Check for Hi on Col1, if Low then Key7 is pressed
        Jmp twelve      ;If Key7 not pressed skip to next key
K7:     Call Key7       ;If key 7 is pressed run subroutine Key7
twelve: JNB Col2, K8    ;Check for Hi on Col2, If Low then Key8 is pressed
        Jmp trteen      ;If key 8 not pressed skip to next key
K8:     Call Key8       ;If key 8 is pressed run subroutine Key8
trteen: JNB Col3, K9    ;Check for Hi on Col3, If Low then Key9 is pressed
        Jmp frteen      ;If key9 not pressed, skip to next key
K9:     Call Key9       ;If Key 9 is pressed run subroutine Key9
frteen: JNB Col4, KD    ;Check for Hi on Col4, If Low then KeyD is pressed
        Jmp ffteen      ;If key D not pressed, skip to next key
KD:     Call KeyD       ;If key D is pressed run subroutine KeyD 
ffteen:	SETB Row3       ;Reinitialise ROW3

;Row 4 (Keys A, 0, B, C) check
        CLR Row4        ;Drive Row4 low (Y4)
sxteen: JNB Col1, kA    ;Check for Hi on Col1, if Low then KeyA is pressed
        Jmp svteen      ;If key A not pressed, skip to next key
KA:     Call KeyA       ;If key A is pressed run subroutine KeyA
svteen: JNB Col2, K0    ;Check for Hi on Col2, If Low then Key0 is pressed
        Jmp etteen      ;If Key 0 not pressed, skip to next key
K0:     Call Key0       ;If key 0 is pressed run subroutine Key0
etteen: JNB Col3, KB    ;Check for Hi on Col3, If Low then KeyB is pressed
        Jmp nnteen      ;If Key B not pressed, skip to next key
KB:     Call KeyB       ;If key B is pressed run subroutine KeyB
nnteen: JNB Col4, KC    ;Check for Hi on Col4, If Low then KeyC is pressed
        Jmp twenty      ;If key C not pressed, skip to next key
KC:     Call KeyC       ;If key C is pressed run subroutine KeyC
twenty: SETB Row4       ;Reinitialise Row4

        JMP Keys        ;Repeat key poll

;**********************************************************************************
;************* SubRoutines ********************************************************
;**********************************************************************************
key1:                   ;Key1 pressed
        MOV A, #1
        Call SendVal
        JNB COL1, $     ;prevent repeated prints if key held		
        Call delay      ;Prevent Debounce
        RET		

Key2:                   ;Key2 pressed 
       	MOV A, #2
        Call SendVal
        JNB COL2, $     ;prevent repeated prints if key held
        Call delay      ;Prevent Debounce
        RET	

Key3:                   ;Key3 pressed
        MOV A, #3
        Call SendVal
        JNB COL3, $     ;prevent repeated prints if key held
        Call delay      ;Prevent Debounce
        RET

KeyF:                   ;KeyF pressed
        MOV A, #0Fh
        Call SendVal
        JNB COL4, $     ;prevent repeated prints if key held
        Call delay      ;Prevent Debounce
        RET	

Key4:                   ;Key4 pressed
        MOV A, #4
        Call SendVal
        JNB COL1, $     ;prevent repeated prints if key held
        Call delay      ;Prevent Debounce
        RET	

Key5:                   ;Key5 pressed
        MOV A, #5
        Call SendVal
        JNB COL2, $     ;prevent repeated prints if key held
        Call delay      ;Prevent Debounce
        RET	

Key6:                   ;Key6 pressed
        MOV A, #6
        Call SendVal
        JNB Col3, $     ;prevent repeated prints if key held
        Call delay      ;Prevent Debounce
        RET	

KeyE:                   ;KeyE pressed
        MOV A, #0Eh
        Call SendVal
        JNB COL4, $     ;prevent repeated prints if key held
        Call delay      ;Prevent Debounce
        RET	

Key7:                   ;Key7 pressed
        MOV A, #7
        Call SendVal
        JNB COL1, $     ;prevent repeated prints if key held
        Call delay      ;Prevent Debounce
        RET	

Key8:                   ;Key8 pressed
        MOV A, #8
        Call SendVal
        JNB COL2, $     ;prevent repeated prints if key held
        Call delay      ;Prevent Debounce
        RET	

Key9:                   ;Key9 pressed
        MOV A, #9
        Call SendVal
        JNB COL3, $     ;prevent repeated prints if key held
        Call delay      ;Prevent Debounce
        RET	

KeyD:                   ;KeyD pressed
        MOV A, #0Dh
        Call SendVal
        JNB COL4, $     ;prevent repeated prints if key held
        Call delay      ;Prevent Debounce
        RET	

KeyA:                   ;KeyA pressed
        MOV A, #0Ah
        Call SendVal
        JNB COL1, $     ;prevent repeated prints if key held
        Call delay      ;Prevent Debounce
        RET	

Key0:                   ;Key0 pressed
        MOV A, #0
        Call SendVal
        JNB COL2, $     ;prevent repeated prints if key held
        Call delay      ;Prevent Debounce
        RET	

KeyB:                   ;KeyB pressed
        MOV A, #0Bh
        Call SendVal
        JNB COL3, $     ;prevent repeated prints if key held
        Call delay      ;Prevent Debounce
        RET	

KeyC:                   ;KeyC pressed
        MOV A, #0Ch
        Call SendVal
        JNB COL4, $     ;prevent repeated prints if key held
        Call delay      ;Prevent Debounce
        RET	
;____________________________________________________________________                 

SENDCHAR:               ;sends ASCII value in A to UART
        MOV SBUF,A
        JNB TI,$        ;wait til present char gone
        CLR TI          ;must clear TI
        RET
;____________________________________________________________________

SENDVAL:                ;converts hex value of A into two 
                        ;ASCII chars, then spits these two 
                        ;chars up the UART. A is unchanged.
        PUSH ACC
        SWAP A
        CALL HEX2ASCII
        CALL SENDCHAR   ;send high nibble
        POP  ACC
        PUSH ACC
        CALL HEX2ASCII
        CALL SENDCHAR   ;send low nibble
        POP  ACC
        RET
;____________________________________________________________________

HEX2ASCII:              ;converts A into the hex char representing
                        ;A's least significant nibble
        ANL  A,#00Fh
        CJNE A,#00Ah,$+3
        JC   IO0030
        ADD  A,#007h
IO0030: ADD  A,#'0'
        RET
;____________________________________________________________________

DELAY:                  ;delay 100ms
        MOV  R7,#200    ;200*500us =100ms..(MOV dest,#data =24clks) 
DLY1:   MOV  R6,#229    ;229*2.17us=500us..(MOV dest,#data =24clks)
        DJNZ R6,$       ;sit here for 500us
        DJNZ R7,DLY1    ;repeat 200 times (100ms delay)
        RET

;____________________________________________________________________

END
